@@ -1,16 +1,18 @@
 import logging
 import threading
+import datetime
 import time
 from functools import wraps
-from typing import Dict, Iterable, List, Optional, Union
+from typing import Dict, Iterable, Optional, Union
 
 import binsync.data
+from binsync.data import ProjectConfig
 from binsync.common.artifact_lifter import ArtifactLifter
 from binsync.core.client import Client, SchedSpeed, Scheduler, Job
 from binsync.data.type_parser import BSTypeParser, BSType
 from binsync.data import (
     State, User, Artifact,
-    Function, FunctionHeader, FunctionArgument, StackVariable,
+    Function, FunctionHeader, StackVariable,
     Comment, GlobalVariable, Patch,
     Enum, Struct
 )
@@ -118,22 +120,26 @@ class BinSyncController:
         self.client = None  # type: Optional[Client]
 
         # ui callback created on UI init
-        self.ui_callback = None  # func()
+        self.ui_callback = None  # func(states: List[State])
         self.ctx_change_callback = None  # func()
         self._last_reload = None
         self.last_ctx = None
-        self._init_ui_components()
+        # ui worker that fires off requests for UI update
+        self._ui_updater_thread = None
+        self._ui_updater_worker: Scheduler = None
 
         # settings
+        self.config = None
+        self.table_coloring_window = 60 * 30  # 30 mins
         self.merge_level: int = MergeLevel.NON_CONFLICTING
 
         # command locks
-        self.job_scheduler = Scheduler()
+        self.push_job_scheduler = Scheduler()
         self.sync_lock = threading.Lock()
 
         # create a pulling thread, but start on connection
-        self.updater_thread = threading.Thread(target=self.updater_routine)
         self._run_updater_threads = False
+        self.user_states_update_thread = threading.Thread(target=self.updater_routine)
 
         # TODO: make the initialization of this with types of decompiler
         self.type_parser = BSTypeParser()
@@ -150,78 +156,113 @@ class BinSyncController:
         from binsync.common.ui.qt_objects import (
             QThread
         )
+        from binsync.common.ui.utils import BSUIScheduler
+        # spawns a qthread/worker
+        self._ui_updater_thread = QThread()
+        self._ui_updater_worker = BSUIScheduler()
+        self._ui_updater_worker.moveToThread(self._ui_updater_thread)
+        self._ui_updater_thread.started.connect(self._ui_updater_worker.run)
+        self._ui_updater_thread.finished.connect(self._ui_updater_thread.deleteLater)
+        self._ui_updater_thread.start()
+
+    def _stop_ui_components(self):
+        if self.headless:
+            return
+
+        #stop the worker, quit the thread, wait for it to exit
+        if self._ui_updater_worker and self._ui_updater_thread:
+            self._ui_updater_worker.stop()
+            self._ui_updater_thread.quit()
+            _l.debug("Waiting for QThread ui_updater_thread to exit..")
+            self._ui_updater_thread.wait()
 
     def schedule_job(self, cmd_func, *args, blocking=False, **kwargs):
         if blocking:
-            return self.job_scheduler.schedule_and_wait_job(
-                Job(cmd_func, *args, **kwargs)
+            return self.push_job_scheduler.schedule_and_wait_job(
+                Job(cmd_func, *args, **kwargs),
+                priority=SchedSpeed.FAST
             )
 
-        self.job_scheduler.schedule_job(
-            Job(cmd_func, *args, **kwargs)
+        self.push_job_scheduler.schedule_job(
+            Job(cmd_func, *args, **kwargs),
+            priority=SchedSpeed.FAST
         )
         return None
 
     def updater_routine(self):
         while self._run_updater_threads:
             time.sleep(BUSY_LOOP_COOLDOWN)
+            now = datetime.datetime.now(tz=datetime.timezone.utc)
 
-            # validate a client is connected to this controller (may not have remote )
+            # validate a client is connected to this controller (which may be local only)
             if not self.check_client():
                 continue
 
             # do git pull/push operations if a remote exist for the client
-            if self.client.last_pull_attempt_ts is None:
+            if self.client.last_pull_attempt_time is None:
                 self.client.update(commit_msg="User created")
 
             # update every reload_time
-            elif time.time() - self.client.last_pull_attempt_ts > self.reload_time:
+            elif (now - self.client.last_pull_attempt_time).seconds > self.reload_time:
                 self.client.update()
-
+                
             if not self.headless:
+                all_states = self.client.all_states()
+                if not all_states:
+                    _l.warning(f"There were no states remote or local.")
+                    continue
+
                 # update context knowledge every loop iteration
                 if self.ctx_change_callback:
-                    self._check_and_notify_ctx()
+                    self._ui_updater_worker.schedule_job(
+                        Job(self._check_and_notify_ctx, all_states)
+                    )
 
                 # update the control panel with new info every BINSYNC_RELOAD_TIME seconds
                 if self._last_reload is None or \
-                        time.time() - self._last_reload > self.reload_time:
-                    self._last_reload = time.time()
-                    self._update_ui()
+                        (now - self._last_reload).seconds > self.reload_time:
+                    self._last_reload = datetime.datetime.now(tz=datetime.timezone.utc)
 
-    def _update_ui(self):
+                    self._ui_updater_worker.schedule_job(
+                        Job(self._update_ui, all_states)
+                    )
+
+    def _update_ui(self, states):
         if not self.ui_callback:
             return
 
-        self.ui_callback()
+        self.ui_callback(states)
 
-    def _check_and_notify_ctx(self):
+    def _check_and_notify_ctx(self, states):
         active_ctx = self.active_context()
         if active_ctx is None or self.last_ctx == active_ctx:
             return
 
         self.last_ctx = active_ctx
-        self.ctx_change_callback()
+        self.ctx_change_callback(states)
 
     def start_worker_routines(self):
         self._run_updater_threads = True
-        self.updater_thread.setDaemon(True)
-        self.updater_thread.start()
+        self.user_states_update_thread.daemon = True
+        self.user_states_update_thread.start()
 
-        self.job_scheduler.start_worker_thread()
+        self.push_job_scheduler.start_worker_thread()
+
+        self._init_ui_components()
 
     def stop_worker_routines(self):
         self._run_updater_threads = False
-        self.job_scheduler.stop_worker_thread()
+        self.push_job_scheduler.stop_worker_thread()
+        self._stop_ui_components()
 
     #
     # Client Interaction Functions
     #
 
-    def connect(self, user, path, init_repo=False, remote_url=None):
+    def connect(self, user, path, init_repo=False, remote_url=None, **kwargs):
         binary_hash = self.binary_hash()
         self.client = Client(
-            user, path, binary_hash, init_repo=init_repo, remote_url=remote_url
+            user, path, binary_hash, init_repo=init_repo, remote_url=remote_url, **kwargs
         )
 
         self.start_worker_routines()
@@ -250,8 +291,8 @@ class BinSyncController:
         self.headless = not self.headless
 
     @init_checker
-    def users(self, priority=None) -> Iterable[User]:
-        return self.client.users(priority=priority)
+    def users(self, priority=None, no_cache=True) -> Iterable[User]:  # TODO: fix no_cache user bug
+        return self.client.users(priority=priority, no_cache=no_cache)
 
     def usernames(self, priority=None) -> Iterable[str]:
         for user in self.users(priority=priority):
@@ -332,7 +373,7 @@ class BinSyncController:
         """
         return {}
 
-    def function(self, addr) -> Optional[Function]:
+    def function(self, addr, **kwargs) -> Optional[Function]:
         return None
 
     def global_vars(self) -> Dict[int, GlobalVariable]:
@@ -557,6 +598,7 @@ class BinSyncController:
             master_artifact,  self.lower_artifact(art_getter(state, *identifiers)),
             merge_level=merge_level, master_state=master_state
         )
+        merged_artifact.last_change = None
 
         lock = self.sync_lock if not self.sync_lock.locked() else FakeSyncLock()
         with lock:
@@ -651,7 +693,7 @@ class BinSyncController:
         """
         Grab all relevant information from the specified user and fill the @func_addr.
         """
-        dec_func: Function = self.function(func_addr)
+        dec_func: Function = self.function(func_addr, **kwargs)
         if not dec_func:
             _l.warning(f"The function at {hex(func_addr)} does not exist in your decompiler.")
             dec_func = Function(None, None)
@@ -662,7 +704,7 @@ class BinSyncController:
         # function header
         if master_func.header and master_func.header != dec_func.header:
             # type is user made (a struct)
-            changes |= self.import_user_defined_type(master_func.header.ret_type, **kwargs)
+            changes |= self.import_user_defined_type(master_func.header.type, **kwargs)
             changes |= self.fill_function_header(func_addr, artifact=master_func.header, **kwargs)
 
         # stack vars
@@ -675,7 +717,7 @@ class BinSyncController:
                 if dec_sv == sv:
                     continue
 
-                corrected_off = dec_sv.stack_offset if dec_sv and dec_sv.stack_offset else sv.stack_offset
+                corrected_off = dec_sv.offset if dec_sv and dec_sv.offset else sv.offset
                 changes |= self.import_user_defined_type(sv.type, **kwargs)
                 changes |= self.fill_stack_variable(func_addr, corrected_off, artifact=sv, **kwargs)
 
@@ -820,6 +862,7 @@ class BinSyncController:
                     pref_art = pref_art.nonconflict_merge(user_art)
                     pref_art.last_change = None
 
+                _l.debug(f"Filling artifact {pref_art} now...")
                 try:
                     filler_func(
                         identifier, artifact=pref_art, state=master_state,  commit_msg=f"Magic Synced {pref_art}",
@@ -939,7 +982,7 @@ class BinSyncController:
         if not type_.is_unknown:
             return None
 
-        base_type_str = type_.base_type.type_str
+        base_type_str = type_.base_type.type
         return base_type_str if base_type_str in state.structs.keys() else None
 
     def import_user_defined_type(self, type_str, **kwargs):
@@ -954,7 +997,7 @@ class BinSyncController:
             return False
 
         nested_undefined_structs = False
-        for off, memb in struct.struct_members.items():
+        for off, memb in struct.members.items():
             user_type = self.type_is_user_defined(memb.type, state=state)
             if user_type and user_type not in master_state.structs.keys():
                 # should we ever happen to have a struct with a nested type that is
@@ -976,3 +1019,31 @@ class BinSyncController:
             or self.get_state(priority=SchedSpeed.FAST)
 
         return master_state, state
+
+    #
+    # Config Utils
+    #
+
+    def load_saved_config(self):
+        config = ProjectConfig.load_from_file(self.binary_path() or "")
+        if not config:
+            return
+        self.config = config
+        _l.info(f"Loaded configuration file: '{self.config.path}'")
+
+        self.config = config
+        self.table_coloring_window = self.config.table_coloring_window or self.table_coloring_window
+        self.merge_level = self.config.merge_level or self.merge_level
+
+        if self.config.log_level == "debug":
+            logging.getLogger("binsync").setLevel("DEBUG")
+            logging.getLogger("ida_binsync").setLevel("DEBUG")
+
+        else:
+            logging.getLogger("binsync").setLevel("INFO")
+            logging.getLogger("ida_binsync").setLevel("INFO")
+
+        return self.config
+
+
+
