@@ -3,7 +3,7 @@ import pathlib
 import os
 import re
 import subprocess
-import time
+import datetime
 from functools import wraps
 from typing import Iterable, Optional
 
@@ -45,7 +45,7 @@ def atomic_git_action(f):
         no_cache = kwargs.get("no_cache", False)
         if not no_cache:
             # cache check
-            cache_item = self._check_cache_(f, **kwargs)
+            cache_item = self.check_cache_(f, **kwargs)
             if cache_item is not None:
                 return cache_item
 
@@ -58,7 +58,7 @@ def atomic_git_action(f):
 
         self._set_cache(f, ret_val, **kwargs)
 
-        return ret_val
+        return ret_val if ret_val is not None else {}
 
     return _atomic_git_action
 
@@ -74,7 +74,10 @@ class Client:
         init_repo: bool = False,
         remote_url: Optional[str] = None,
         ssh_agent_pid: Optional[int] = None,
-        ssh_auth_sock: Optional[str] = None
+        ssh_auth_sock: Optional[str] = None,
+        push_on_update=True,
+        pull_on_update=True,
+        **kwargs,
     ):
         """
         The Client class is responsible for making the low-level Git operations for the BinSync environment.
@@ -97,9 +100,11 @@ class Client:
         self.remote = remote
         self.repo = None
         self.repo_lock = None
+        self.pull_on_update = pull_on_update
+        self.push_on_update = push_on_update
 
         # validate this username can exist
-        if master_user.endswith('/') or '__root__' in master_user:
+        if not master_user or master_user.endswith('/') or '__root__' in master_user:
             raise Exception(f"Bad username: {master_user}")
 
         # ssh-agent info
@@ -118,11 +123,11 @@ class Client:
 
         # timestamps
         self._commit_interval = commit_interval
-        self._last_push_ts = None  # type: Optional[float]
-        self.last_push_attempt_ts = None  # type: Optional[float]
-        self._last_pull_ts = None  # type: Optional[float]
-        self.last_pull_attempt_ts = None  # type: Optional[float]
-        self._last_commit_ts = None # type: Optional[float]
+        self._last_push_time = None  # type: datetime.datetime
+        self.last_push_attempt_time = None  # type: datetime.datetime
+        self._last_pull_time = None  # type: datetime.datetime
+        self.last_pull_attempt_time = None  # type: datetime.datetime
+        self._last_commit_time = None # type: datetime.datetime
 
         # load or update the global binsync config
         self.global_config = self._load_or_update_config()
@@ -139,7 +144,7 @@ class Client:
 
     def _load_or_update_config(self):
         config = GlobalConfig.load_from_file(None) or GlobalConfig(None)
-        config.last_bs_repo_path = self.repo_root
+        config.add_recent_project_path(self.repo_root, self.master_user)
         config.save()
         return config
 
@@ -245,15 +250,15 @@ class Client:
 
     @property
     def last_push_ts(self):
-        return self._last_push_ts
+        return self._last_push_time
 
     @property
     def last_pull_ts(self):
-        return self._last_pull_ts
+        return self._last_pull_time
 
     @property
     def last_commit_ts(self):
-        return self._last_commit_ts
+        return self._last_commit_time
 
     @property
     def user_branch_name(self):
@@ -287,7 +292,7 @@ class Client:
         except Exception as e:
             l.warning(f"Internal Git Commit Error: {e}")
             return
-        self._last_commit_ts = time.time()
+        self._last_commit_time = datetime.datetime.now(tz=datetime.timezone.utc)
         master_user_branch.commit = commit
         state._dirty = False
 
@@ -296,6 +301,7 @@ class Client:
         repo = self.repo
         users = list()
         for ref in self._get_best_refs(repo).values():
+            l.debug(f"{ref} NAME: {ref.name}")
             try:
                 metadata = load_toml_from_file(ref.commit.tree, "metadata.toml", client=self)
                 user = User.from_metadata(metadata)
@@ -303,7 +309,7 @@ class Client:
             except Exception as e:
                 l.debug(f"Unable to load user {e}")
                 continue
-
+        l.debug(users)
         return users
 
     @atomic_git_action
@@ -312,7 +318,7 @@ class Client:
             user = self.master_user
 
         repo = self.repo
-        state = None
+        state = State(None)
         try:
             state = State.parse(
                 self._get_tree(user, repo),
@@ -332,7 +338,7 @@ class Client:
 
         :return:    None
         """
-        self.last_pull_attempt_ts = time.time()
+        self.last_pull_attempt_time = datetime.datetime.now(tz=datetime.timezone.utc)
         try:
             env = self.ssh_agent_env()
         except Exception:
@@ -345,10 +351,10 @@ class Client:
                 self.repo.git.checkout(BINSYNC_ROOT_BRANCH)
                 self.repo.git.fetch("--all")
                 self.repo.git.pull("--all")
-                self._last_pull_ts = time.time()
+                self._last_pull_time = datetime.datetime.now(tz=datetime.timezone.utc)
                 self.active_remote = True
             except Exception as e:
-                l.debug(f"Pull exception {e}")
+                #l.debug(f"Pull exception {e}")
                 self.active_remote = False
 
         if not self.active_remote:
@@ -375,19 +381,19 @@ class Client:
 
         :return:    None
         """
-        self.last_push_attempt_ts = time.time()
+        self.last_push_attempt_time = datetime.datetime.now(tz=datetime.timezone.utc)
         self._checkout_to_master_user()
         try:
             env = self.ssh_agent_env()
             with self.repo.git.custom_environment(**env):
                 self.repo.remotes[self.remote].push(BINSYNC_ROOT_BRANCH)
                 self.repo.remotes[self.remote].push(self.user_branch_name)
-            self._last_push_ts = time.time()
+            self._last_push_time = datetime.datetime.now(tz=datetime.timezone.utc)
             #l.debug("Push completed successfully at %s", self._last_push_ts)
             self.active_remote = True
         except git.exc.GitCommandError as ex:
             self.active_remote = False
-            l.debug(f"Failed to push b/c {ex}")
+            #l.debug(f"Failed to push b/c {ex}")
 
     @property
     @atomic_git_action
@@ -403,6 +409,14 @@ class Client:
     # Non-Atomic Public API
     #
 
+    def all_states(self):
+        states = list()
+        for user in self.users(no_cache=True):
+            state = self.get_state(user=user.name)
+            states.append(state)
+
+        return states
+
     def update(self, commit_msg="Generic Change"):
         """
         Update both the local and remote repo knowledge of files through pushes/pulls and commits
@@ -414,12 +428,12 @@ class Client:
             self.commit_state(master_state, msg=commit_msg)
 
         # do a pull if there is a remote repo connected
-        self.last_pull_attempt_ts = time.time()
-        if self.has_remote:
+        self.last_pull_attempt_time = datetime.datetime.now(tz=datetime.timezone.utc)
+        if self.has_remote and self.pull_on_update:
             self.pull()
 
-        self.last_push_attempt_ts = time.time()
-        if self.has_remote:
+        self.last_push_attempt_time = datetime.datetime.now(tz=datetime.timezone.utc)
+        if self.has_remote and self.push_on_update:
             self.push()
 
     #
@@ -645,7 +659,7 @@ class Client:
         }
         return commit_dict
 
-    def _check_cache_(self, f, **kwargs):
+    def check_cache_(self, f, **kwargs):
         if f.__qualname__ == self.get_state.__qualname__:
             cache_func = self.cache.get_state
             args = []
