@@ -52,7 +52,7 @@ class Scheduler:
 
     def start_worker_thread(self):
         self._work = True
-        self._worker.setDaemon(True)
+        self._worker.daemon = True
         self._worker.start()
 
     def _worker_thread(self):
@@ -60,6 +60,10 @@ class Scheduler:
             self._complete_a_job(block=True)
 
     def schedule_job(self, job: Job, priority=SchedSpeed.SLOW):
+        if not self._work:
+            l.warning(f"The job scheduler is not currently set to work, but you are still scheduling a job...")
+
+        #l.debug(f"Job scheduled: {job} with priority {priority}")
         self._job_queue.put_nowait(
             (priority, job,)
         )
@@ -81,4 +85,5 @@ class Scheduler:
         else:
             return
 
+        #l.debug(f"Completing scheduled job now: {job}")
         job.execute()
