@@ -3,10 +3,10 @@ from binsync.data.artifact import Artifact
 from binsync.data.func import Function, FunctionHeader, FunctionArgument
 from binsync.data.comment import Comment
 from binsync.data.patch import Patch
-from binsync.data.stack_variable import StackVariable, StackOffsetType
+from binsync.data.stack_variable import StackVariable
 from binsync.data.struct import StructMember, Struct
 from binsync.data.global_variable import GlobalVariable
 from binsync.data.enum import Enum
-from binsync.data.state import State
+from binsync.data.state import State, ArtifactType
 from binsync.data.configuration import ProjectConfig, GlobalConfig
 from binsync.data.type_parser import BSType, BSTypeParser
