@@ -7,19 +7,19 @@ class GlobalVariable(Artifact):
     __slots__ = Artifact.__slots__ + (
         "addr",
         "name",
-        "type_str",
+        "type",
         "size"
     )
 
-    def __init__(self, addr, name, type_str=None, size=0, last_change=None):
+    def __init__(self, addr, name, type_=None, size=0, last_change=None):
         super(GlobalVariable, self).__init__(last_change=last_change)
         self.addr = addr
         self.name = name
-        self.type_str = type_str
+        self.type = type_
         self.size = size
 
     def __str__(self):
-        return f"<GlobalVar: {self.type_str} {self.name}; @{self.addr} len={self.size}>"
+        return f"<GlobalVar: {self.type} {self.name}; @{self.addr} len={self.size}>"
 
     def __repr__(self):
         return self.__str__()
@@ -46,9 +46,9 @@ class GlobalVariable(Artifact):
         global_vars_ = {}
 
         for v in sorted(global_vars.values(), key=lambda x: x.addr):
-            global_vars_["%x" % v.addr] = v.__getstate__()
+            global_vars_[hex(v.addr)] = v.__getstate__()
         return global_vars_
 
     def copy(self):
-        gvar = GlobalVariable(self.addr, self.name, self.type_str, self.size)
+        gvar = GlobalVariable(self.addr, self.name, type_=self.type, size=self.size, last_change=self.last_change)
         return gvar
